

//-----------------------------------------------------------------------------
// GPIO COMMANDS --------------------------------------------------------------
//-----------------------------------------------------------------------------


//description:   Turn module on
//input:         none
//return:        none
void gpio_turnOn() {
  debug_print1( "GPIO: TurnOn",             LEVEL_INFO );
  delay(100);
  digitalWrite(pin_po,  LOW);
  delay(100);
  digitalWrite(pin_po,  HIGH);
}

byte gpio_ledRead(byte ledNum) {
  //This yields a resolution between readings of: 5 volts / 1024 units or, .0049 volts (4.9 mV) per unit
  int led = analogRead(ledNum);       // read the input pin
  debug_printLevel(LEVEL_DEBUG);
  debug_print( "GPIO: ledRead: ",           LEVEL_DEBUG );
  debug_print( String(ledNum),              LEVEL_DEBUG );
  debug_print( " - ",                       LEVEL_DEBUG );
  debug_print( String(led),                 LEVEL_DEBUG );
  debug_println( LEVEL_DEBUG );
  if (led > LED_THRESHOLD) return 1;
  return 0;
}

//description:   Check if module is connected to a network or not
//input:         none
//return:        0=2G or 3G, 1=unregistered
byte gpio_ledConnected() {
  debug_print1( "GPIO: LED Connected",      LEVEL_INFO );
  byte gpio_ledB = gpio_ledRead(pin_ledB);                      //3G
  //byte gpio_ledR = gpio_ledRead(pin_ledR);                      //Unregistered
  byte gpio_ledG = gpio_ledRead(pin_ledG);                      //2G
  if ((gpio_ledG == 0) or (gpio_ledB == 0)) {
    debug_print1( " Connected to a cellular network", LEVEL_INFO );
    return 0;
  }
  else {
    debug_print1( " SIM Unregistered",      LEVEL_INFO );
    return 1;
  }
}
        
//description:   Check if module power is on (one of the LEDs should be lit with power)
//input:         none
//return:        0=power,1=off
byte gpio_ledLit() {
  debug_print1( "GPIO: LED Lit",            LEVEL_INFO );
  byte gpio_ledB = gpio_ledRead(pin_ledB);                      //3G
  byte gpio_ledR = gpio_ledRead(pin_ledR);                      //Unregistered
  byte gpio_ledG = gpio_ledRead(pin_ledG);                      //2G
  
  if ((gpio_ledG == 0) or (gpio_ledB == 0) or (gpio_ledR == 0)) {
    debug_print1( " On",                    LEVEL_INFO );
    return 0;
  }
  else {
    debug_print1( " Off",                   LEVEL_INFO );
    return 1;
  }
}
        
//-----------------------------------------------------------------------------
// DEBUG COMMANDS -------------------------------------------------------------
//-----------------------------------------------------------------------------

//description:   Print state of network LEDs
//input:         none
//return:        none
void debug_led() {
  debug_print1( "LED",                      LEVEL_DEBUG );
  byte ledR = gpio_ledRead(pin_ledR);
  byte ledG = gpio_ledRead(pin_ledG);
  byte ledB = gpio_ledRead(pin_ledB);
  
  debug_printLevel(LEVEL_DEBUG);
  debug_print( " ledR: ",                   LEVEL_DEBUG );
  debug_print( String(ledR),                LEVEL_DEBUG );
  debug_print( " Red   - Not Connected (0 on, 1 off)", LEVEL_DEBUG );
  debug_println( LEVEL_DEBUG );
  
  debug_printLevel(LEVEL_DEBUG);
  debug_print( " ledG: ",                   LEVEL_DEBUG );
  debug_print( String(ledG),                LEVEL_DEBUG );
  debug_print( " Green - 2G (0 on, 1 off)", LEVEL_DEBUG );
  debug_println( LEVEL_DEBUG );
  
  debug_printLevel(LEVEL_DEBUG);
  debug_print( " ledB: ",                   LEVEL_DEBUG );
  debug_print( String(ledB),                LEVEL_DEBUG );
  debug_print( " Blue  - 3G (0 on, 1 off)", LEVEL_DEBUG );
  debug_println( LEVEL_DEBUG );
}

//description:   Print state of status LEDs
//input:         none
//return:        none
void debug_inputs() {
  debug_print1( "Inputs",                   LEVEL_DEBUG );
  
  debug_printLevel(LEVEL_DEBUG);
  debug_print( " Module Wakeup AP: ",       LEVEL_DEBUG );
  debug_print( String(digitalRead(pin_mwa)),LEVEL_DEBUG );
  debug_print( "(0 wake AP, 1 AP awake)",   LEVEL_DEBUG );
  debug_println( LEVEL_DEBUG );
    
  debug_printLevel(LEVEL_DEBUG);
  debug_print( " Module Poweron: ",         LEVEL_DEBUG );
  debug_print( String(digitalRead(pin_mp)), LEVEL_DEBUG );
  debug_print( "(0 off, 1 power)",          LEVEL_DEBUG );
  debug_println( LEVEL_DEBUG );
    
  debug_printLevel(LEVEL_DEBUG);
  debug_print( " Module Ready: ",           LEVEL_DEBUG );
  debug_print( String(digitalRead(pin_mr)), LEVEL_DEBUG );
  debug_print( "(0 awake, 1 asleep)",       LEVEL_DEBUG );
  debug_println( LEVEL_DEBUG );
}
    
//description:   debug print function - debug level for long statements
//input:         none
//return:        none
void debug_printLevel(byte level) {
  if (level <= glob_debug) {
    if (LEVEL_CRITICAL == level)     mySerial.print("CRITICAL:  ");
    else if (LEVEL_ERROR == level)   mySerial.print("ERROR:     ");
    else if (LEVEL_WARNING == level) mySerial.print("WARNING:   ");
    else if (LEVEL_INFO == level)    mySerial.print("INFO:      ");
    else if (LEVEL_DEBUG == level)   mySerial.print("DEBUG:     ");
    else                             mySerial.print("UNDEFINED: ");
  }
}
//description:   debug print function - single string for long statements
//input:         none
//return:        none
void debug_print(String string, byte level) {
  if (level <= glob_debug) {
    mySerial.print(string);
  }
}
//description:   debug print function - newline for long strings
//input:         none
//return:        none
void debug_println(byte level) {
  if (level <= glob_debug) {
    mySerial.println("");
  }
}

//description:   debug print function - print everything for short statements
//input:         none
//return:        none
void debug_print1(String string, byte level) {
  debug_printLevel(level);
  if (level <= glob_debug) {
    mySerial.println(string);
  }
}

//-----------------------------------------------------------------------------
// PROCESSING COMMANDS --------------------------------------------------------
//-----------------------------------------------------------------------------

//description:   Setup GPIO for RasPi shield
//input:         none
//return:        none
void cmd_setup() {
  debug_print1( "CMD Setup",                LEVEL_DEBUG );
  Serial.begin(115200);
  
  pinMode(pin_prn, OUTPUT);
  pinMode(pin_mwa, INPUT);
  pinMode(pin_mp,  INPUT);
  pinMode(pin_mr,  INPUT);
  pinMode(pin_ar,  OUTPUT);
  pinMode(pin_awm, OUTPUT);
  pinMode(pin_po,  OUTPUT);
  
  digitalWrite(pin_prn, HIGH);            //no reset (toggle to reset)
  digitalWrite(pin_ar,  LOW);             //AP is awake
  digitalWrite(pin_awm, LOW);             //AP wakes up module
  digitalWrite(pin_po,  HIGH);            //high is awake (toggle later to turn on/off)
}
    
//description    Check if CELLv1.0 is on, if not, turn it on
//input:         none
//return:        none
byte cmd_power() {
  debug_print1( "CMD Power",                LEVEL_DEBUG );
    byte tries = 3;
    byte ret = 1;
    while (tries > 0) {
        tries -= 1;
        debug_led();
        debug_inputs();
        
        byte tries_led=3;                 //in case the LED is flashing
        while ((tries_led > 0) and (ret != 0)) {
            ret &= gpio_ledLit();
            tries_led -= 1;
            delay(1000);
        }
        
        debug_printLevel(LEVEL_DEBUG);
        debug_print( "LED RETURN: ",        LEVEL_DEBUG );
        debug_print( String(ret),           LEVEL_DEBUG );
        debug_print( "(0 1+ LED on, 1 all LEDs off)", LEVEL_DEBUG );
        debug_println( LEVEL_DEBUG );
        
        if (ret != 0) {
            //ret_status = gpio_statusMR();
            //if (ret_status != 0)
            gpio_turnOn();
        }
        else {
            tries = 0;
        }
    }
    if (ret != 0) {
        debug_print1( "FATAL ERROR - Module is not turning on (Check power)", LEVEL_CRITICAL );
              
        //KILLS APP *****
        //while (1);
      
        return 1;
    }
    return 0;
}

